/* -*- C++ -*- */

#ifndef _SRItclStar_h
#define _SRItclStar_h

/*  Version @(#)SRItclStar.h	1.1 4/23/96

@Copyright (c) 1996-1997 The Regents of the University of California.
All rights reserved.

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the
above copyright notice and the following two paragraphs appear in all
copies of this software.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

						PT_COPYRIGHT_VERSION_2
						COPYRIGHTENDKEY

    Author:     S. A. Edwards
    Created:    22 April 1996

*/

#ifdef __GNUG__
#pragma interface
#endif

#include "SRNonStrictStar.h"
#include "StringState.h"

extern "C" {
#include "tcl.h"
#include "itcl.h"
}

class PortHole;
class Star;
class HashTable;

/**********************************************************************

  Connects Itcl objects to SRNonStrictStars

  @Description  This class implements a two-way connection between objects
  in itcl and SRNonStrictStar objects.

  <P> For communication from itcl to C++, a series of commands are registered
  with the itcl interpreter (see initialize) that, when called, determine
  the name of "this" in the itcl domain (the object name), looks this up
  in the hash table starTable to get a pointer to the C++ object (see
  startOfCurrentObject), and call the appropriate method on the class.

  <P> For communication from C++ to itcl, the objectName string
  contains the name of the itcl object the star is attached to.
  Methods like go() call the "go" method in the itcl domain on this
  object.

 ***********************************************************************/

class SRItclStar : public SRNonStrictStar {

public:

  SRItclStar();

  ~SRItclStar();

  static int initialize( Tcl_Interp * initializingInterp );

  static SRItclStar * starOfCurrentObject( Tcl_Interp * interp);

  int construct( const char * className, const char * newObjectName,
		 const char * sourceFile, int argc = 0, char *argv[] = 0);

  int destroy();

  int readPort( PortHole * port );

  int writePort( PortHole * port, char * data );

protected:

  void begin();
  void go();
  void tick();

  void wrapup();
  
  // The name of the itcl class associated with this star
  StringState itclClassName;

  // The name of the itcl source file associated with this star
  StringState itclSourceFile;

  // The name of the itcl object to create for this star
  StringState itclObjectName;
  
  // Name of this interface's corresponding Itcl object
  char * objectName;

  // This interfaces's corresponding Itcl object
  Itcl_Object * itclObject;

  // Interpreter in which all objects reside
  static Tcl_Interp * interp;

  // Maps itcl object names to ItclStarIfc *'s
  static HashTable starTable;  

private:

  // Flag indicating whether the system has been initialized
  static int hasInitialized;

};

#endif
