/* -*-C++-*-
 * ###################################################################
 *  SRCGC - Synchronous/Reactive C code generation for Ptolemy
 * 
 *  FILE: "SRCGCNonStrictStar.h"
 *                                    created: 9/03/98 15:08:37 
 *                                last update: 13/05/98 9:21:38 
 *  Author: Vincent Legrand, Mathilde Roger, Frdric Boulanger
 *  E-mail: Frederic.Boulanger@supelec.fr
 *    mail: Suplec - Service Informatique
 *          Plateau de Moulon, F-91192 Gif-sur-Yvette cedex
 *     www: http://wwwsi.supelec.fr/
 *  
 *  Thomson: Xavier Warzee <XAVIER.W.X.WARZEE@tco.thomson.fr>
 *  
 * Copyright (c) 1998 Suplec & Thomson-CSF Optronique.
 * All rights reserved.
 * 
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in all
 * copies of this software.
 * 
 * IN NO EVENT SHALL SUPELEC OR THOMSON-CSF OPTRONIQUE BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 * SUPELEC OR THOMSON-CSF OPTRONIQUE HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * SUPELEC AND THOMSON-CSF OPTRONIQUE SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND SUPELEC OR THOMSON-CSF
 * OPTRONIQUE HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 * ENHANCEMENTS, OR MODIFICATIONS.
 * 
 *  Description: 
 *  This file is really a workaround for ptlang, which insists
 *  on a different header file for each class (see the "derivedfrom"
 *  rule).
 * 
 *  Non-strict "template" star for the SRCGC domain
 *
 *  @Description A non-strict star can always fire, not just when all its
 *  inputs are known.  Strict stars (i.e., those derived from the SRCGCStar class)
 *  are easier to code and use, but may cause deadlock when placed in
 *  zero-delay feedback loops.
 *
 *  History
 * 
 *  modified by  rev reason
 *  -------- --- --- -----------
 *  9/03/98  FBO 1.0 original
 * ###################################################################
 */

#ifndef _SRCGCNonStrictStar_h
#define _SRCGCNonStrictStar_h

#ifdef __GNUG__
#pragma interface
#endif

#include "SRCGCStar.h"

class SRCGCNonStrictStar : public SRCGCStar {
  public:
   // Identify the class
   int isA(const char*) const;

   // Identify the domain
   const char* domain() const;

   int run();
};

#endif
